// miguel 1.04 12:00 latest with OLED
// working example with TTGO screen

#include <TFT_eSPI.h> // Graphics and font library for ST7735 driver chip
#include <SPI.h>
#include "opel.h"
#include "BluetoothSerial.h"
#include "ELMduino.h"
#include "WiFi.h"
#include "Free_Fonts.h"

#define ELM_PORT SerialBT
#define ESP_BLUETOOTH_NAME "ESP32"

BluetoothSerial SerialBT;
ELM327 myELM327;

TFT_eSPI tft = TFT_eSPI(135, 240); // Invoke custom library

void setup() {
  delay(100);
  Serial.begin(115200);

  ELM_PORT.begin(ESP_BLUETOOTH_NAME, true);

  Serial.println("Attempting to connect to ELM327...");

  tft.init();tft.setRotation(3);
  tft.setSwapBytes(true);
  tft.pushImage(0, 0, 240, 135, opel);
  delay(1000);
  
  // change "V-LINK" to your OBD reader bluetooth name //
  if (!ELM_PORT.connect("V-LINK"))
  {
    Serial.println("Couldn't connect to OBD scanner - Phase 1");

      tft.fillScreen(TFT_BLACK);
      tft.setTextColor(TFT_WHITE, TFT_BLACK);
      tft.setCursor(10,40);
      tft.setTextFont(4);
      tft.println("OBD error 1");     
      delay(2000);
      
    // let it not connect & retest connection at the end of the loop
    //while (1);
  }

  if (!myELM327.begin(ELM_PORT))
  {
    Serial.println("Couldn't connect to OBD scanner - Phase 2");
    
      tft.fillScreen(TFT_BLACK);
      tft.setTextColor(TFT_WHITE, TFT_BLACK);
      tft.setCursor(10,40);
      tft.setTextFont(4);
      tft.println("OBD error 1");    

    delay(2000);
    //while (1);
  }
  
  Serial.println("Connected to ELM327");
}


void loop() {

  int32_t rpm = -1;
  int32_t soot = -1;
  int32_t burn = -1;
  int32_t km = -1;
  int32_t litersLeft = -1;
  

// read values first, then display

// read RPM
  float tempRPM = myELM327.rpm();
  if (myELM327.status == ELM_SUCCESS)
  {
    rpm = (int32_t)tempRPM;
    Serial.print("RPM: "); Serial.println(rpm);
  }
  else
      printError();

// read DPF soot level
  if (myELM327.queryPID(34, 13162))
  {
    int32_t tempSoot = myELM327.findResponse();
    
    if (myELM327.status == ELM_SUCCESS)
    {
      soot = tempSoot;
      Serial.print("DPF soot: "); Serial.println(soot);
    }
    else
      printError();
  }

// read DPF km since last regen
  if (myELM327.queryPID(34, 12345))
  {
    int32_t tempKm = myELM327.findResponse();    

    if (myELM327.status == ELM_SUCCESS)
    {
      km = tempKm;
      Serial.print("DPF km: "); Serial.println(km);
    }
    else 
      printError();
  }

// read fuel liters left 
  if (myELM327.queryPID(34, 4906))
  {
    int32_t tempLitersLeft = myELM327.findResponse() / 64.0;    

    if (myELM327.status == ELM_SUCCESS)
    {
      litersLeft = tempLitersLeft;
      Serial.print("Liters: "); Serial.println(litersLeft);
    }
    else
      printError();
  }

// read DPF burn status 0/1
  if (myELM327.queryPID(34, 8434))
  {
    int32_t tempBurn = myELM327.findResponse();

    if (myELM327.status == ELM_SUCCESS)
    {
      burn = tempBurn;
      Serial.print("DPF burns: "); Serial.println(burn);
    }
    else
      printError();
  }

// reading ends, now display it all / blink screen

    if if (burn > 0 && burn <11) {
      dpfBlink(soot);
    } else {
      oneScreen(soot, km, burn, litersLeft, rpm);
      delay(1000);
      // adjust the refresh rate here, mainly visible in rpm update rate
    }

  // check if it's still connected after the loop
  if (!myELM327.connected)
  {
    Serial.println("Lost connection.."); 
      tft.fillScreen(TFT_BLACK);
      tft.setTextColor(TFT_WHITE, TFT_BLACK);
      tft.setFreeFont(FF18);  
      //tft.setTextFont(4);  
      tft.setCursor(10,40);
      tft.println("Lost connection.."); 
    
    // try to reconnect
    ELM_PORT.connect("V-LINK");    
    Serial.println("reconnecting..");
      tft.fillScreen(TFT_BLACK);
      tft.setTextColor(TFT_WHITE, TFT_BLACK);
      tft.setFreeFont(FF18);
      tft.setCursor(10,40);
      //tft.setTextFont(4);
      tft.println("reconnecting.."); 
    delay(500);
    
    myELM327.begin(ELM_PORT);    
    // ESP.restart();
  }  
  
  
}

void dpfBlink(uint8_t dpfSoot)
{
  // Serial.println("aaah! DPF burns!");
  // display 2 blinking screens
              tft.fillScreen(TFT_BLACK);
              tft.setTextColor(TFT_WHITE, TFT_BLACK);

              tft.setCursor(40,40);          
              tft.setFreeFont(FF18);     
              tft.println("DPF burns!"); 
            
              tft.setFreeFont(FF17);
              tft.setCursor(20,70); 
              tft.println("don't turn the engine off");
              
              tft.setFreeFont(FF19); 
              tft.setCursor(40,110); 
              tft.print("Soot :");tft.print(String(dpfSoot));
              tft.drawString("%", 140, 110);

              // screen blink 
              for (int a = 0; a < 13; ++a) {
              tft.invertDisplay(true);
              delay(200);
              tft.invertDisplay(false);
              delay(200);  
              }
              // screen blink

              tft.fillScreen(TFT_BLACK);
              tft.setTextColor(TFT_WHITE, TFT_BLACK);

              tft.setCursor(30,40);
              tft.setTextSize(1);

              tft.setFreeFont(FF18); 
              tft.println("DPF soot level"); 

              tft.setFreeFont(FF24); 
              tft.drawString(String(dpfSoot), 60, 75);
              tft.drawString("%", 120, 75);
  
              // screen blink
              for (int a = 0; a < 12; ++a) {
              tft.invertDisplay(true);
              delay(200);
              tft.invertDisplay(false);
              delay(200);  
              }
              // screen blink  
              
    tft.invertDisplay(true);       
    // Serial.println("DPF blink ended");
}

void oneScreen (uint8_t dpfSoot, uint16_t dpfKm, uint8_t dpfBurn, uint8_t fuel, uint16_t rpm){

      if (dpfSoot > 130) {dpfSoot = 0;}
      if (dpfKm > 999) {dpfKm = 0;}
      if (dpfBurn > 11) {dpfBurn = 0;}
      if (fuel > 100) {fuel = 0;}
      if (rpm > 7000) {rpm = 0;}
      
  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_WHITE, TFT_BLACK);

  tft.setFreeFont(NULL);

  tft.drawString("DPF soot :", 10, 5, 4);
  tft.drawString(String(dpfSoot), 130, 5, 4);
        
  tft.drawString("DPF km :", 10, 31, 4);
  tft.drawString(String(dpfKm), 130, 31, 4);
          
  tft.drawString("fuel :", 10, 57, 4);
  tft.drawString(String(fuel), 130, 57, 4);
        
  tft.drawString("rpm :", 10, 83, 4);
  tft.drawString(String(rpm), 130, 83, 4);

  tft.drawString(String(dpfBurn), 220, 125, 1);
  
}

void printError()
{
  if (myELM327.status == ELM_SUCCESS) 
    Serial.println(F("\tELM_SUCCESS"));
  else if (myELM327.status == ELM_NO_RESPONSE)
    Serial.println(F("\tERROR: ELM_NO_RESPONSE"));
  else if (myELM327.status == ELM_BUFFER_OVERFLOW)
    Serial.println(F("\tERROR: ELM_BUFFER_OVERFLOW"));
  else if (myELM327.status == ELM_GARBAGE)
    Serial.println(F("\tERROR: ELM_GARBAGE"));
  else if (myELM327.status == ELM_UNABLE_TO_CONNECT)
    Serial.println(F("\tERROR: ELM_UNABLE_TO_CONNECT"));
  else if (myELM327.status == ELM_NO_DATA)
    Serial.println(F("\tERROR: ELM_NO_DATA"));
  else if (myELM327.status == ELM_STOPPED)
    Serial.println(F("\tERROR: ELM_STOPPED"));
  else if (myELM327.status == ELM_TIMEOUT)
    Serial.println(F("\tERROR: ELM_TIMEOUT"));
  else if (myELM327.status == ELM_GENERAL_ERROR)
    Serial.println(F("\tERROR: ELM_GENERAL_ERROR")); 

  delay(500);
}
